/*
Cour: STIC 4
Description: libraire permettant de faire un kit de chimie
Auteur: Victor Garretas Ruiz
*/

//Importation de la librairie DobloFactory permettant à nos pièces d'être posées sur des Legos
include <../DobloFactory/doblo-factory.scad>;
include <../DobloFactory/lib/doblo-params.scad>;


/*********************************
Table
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre imprimer sert à placer les pièces de façon à imprimer facilement la table
    Valeurs acceptées: true ou false
*/
module table_lego(lego=true, imprimer=true){
    if(lego == true){
        //base lego
        doblo(col=-9, row=-4, width=18, length=8, height=HALF, nibbles_on_off=false);
        //haut
        color("red"){
            block(col=-9, row=-4, width=10, length=8, height=HALF, up=6.5*FULL, nibbles_on_off=true);
        }
        //Table
        constructionTable();
        if(imprimer == false){
            //Robinet
            translate([40,22.5,60]){ 
                robinet();
            }
        }
        if(imprimer == true){
            //Robinet
            translate([100,0,3]){ 
                rotate(a = [-90,0,0]){
                    robinet();
                }
            }
        }
    }
    if(lego == false){    
        //base lego
        block(col=-9, row=-4, width=18, length=8, height=HALF, nibbles_on_off=false);
        //Table
        constructionTable();
        if(imprimer == false){
            //Robinet
            translate([40,22.5,60]){ 
                robinet();
            }
        }
        if(imprimer == true){
            //Robinet
            translate([100,0,3]){ 
                rotate(a = [-90,0,0]){
                    robinet();
                }
            }
        }
    }    
    
    module constructionTable(){
        //table
        difference(){
            union(){
               //Base de la table
               color("green"){ block(col=-9, row=-4, width=18, length=8, height=6*FULL, up=HALF, nibbles_on_off=false);}
               //Haut de la table
                hull(){
                    color("yellow"){
                        block(col=-10, row=-5, width=20, length=10, up=6*FULL, height=FULL, nibbles_on_off=false);
                        support (col=-9, row=4, height=HALF+1, up=5.5*FULL-1, angle=270, thickness=18, scale=LUGO);
                        support (col=9, row=-4, height=HALF+1, up=5.5*FULL-1, angle=0, thickness=8, scale=LUGO);
                        support (col=-9, row=-5, height=HALF+1, up=5.5*FULL-1, angle=90, thickness=18, scale=LUGO);
                        support (col=-10, row=-4, height=HALF+1, up=5.5*FULL-1, angle=180, thickness=8, scale=LUGO);
                    }
               }
            }//union
            //Lavabo
            color("blue"){
                block(col=2, row=-2, width=6, length=4.5, height=3*FULL, up=5*FULL, nibbles_on_off=false);
            }
            //On fait un trou pour le robinet
            translate([40,22.5,60]){ 
                robinet();
            }
         }//difference
         
        //Tirroirs
        translate ([0,-4.5,-2.2]){
            union(){
                //haut
                translate([28,-24,44]){
                    cube(size =[48,8,16], center = true);  
                }
                translate([28,-26,44]){
                    rotate(a = [90,0,0]){
                       cylinder(h = 4, r = 1);
                    }
                }
                translate([-28,-24,44]){
                    cube(size =[48,8,16], center = true);  
                }
                translate([-28,-26,44]){
                    rotate(a = [90,0,0]){
                       cylinder(h = 4, r = 1);
                    }
                }
                
                //Bas
                translate([40,-24,20]){
                    cube(size =[22,8,24], center = true);
                }
                translate([32,-26,20]){
                    rotate(a = [90,0,0]){
                       cylinder(h = 4, r = 1);
                    }
                }
                translate([16,-24,20]){
                    cube(size =[22,8,24], center = true);
                }
                translate([24,-26,20]){
                    rotate(a = [90,0,0]){
                       cylinder(h = 4, r = 1);
                    }
                }
                translate([-40,-24,20]){
                    cube(size =[22,8,24], center = true);
                }
                translate([-32,-26,20]){
                    rotate(a = [90,0,0]){
                       cylinder(h = 4, r = 1);
                    }
                }        
                translate([-16,-24,20]){
                    cube(size =[22,8,24], center = true);
                }
                translate([-24,-26,20]){
                    rotate(a = [90,0,0]){
                       cylinder(h = 4, r = 1);
                    }
                }        
            }
        }
    }
    //Robinet
    module robinet(){
        //Tronc
        translate([0,0,0]){
            cylinder(h = 30, r = 3);
        }
        //Robinet Gauche
        translate([-19,0,27.5]){
            rotate(a = [0,90,0]){
                cylinder(h = 20, r = 2.5, $fn=15);
            }
        }
        translate([-19,0,27.5]){
            sphere(r = 2.5, $fn=15);
        }
        translate([-19,-20,27.5]){
            rotate(a = [0,90,90]){
                cylinder(h = 20, r = 2.5, $fn=15);
            }    
        } 
        translate([-19,-20,27.5]){
            sphere(r = 2.5, $fn=15);
        }             
        translate([-19,-20,20]){
            rotate(a = [0,0,0]){
                cylinder(h = 7.5, r = 2.5, $fn=15);
            }    
        }
        translate([-19,-17.5,27.5]){
            rotate(a = [0,0,0]){
                cylinder(h = 5, r = 2, $fn=15);
            }    
        }
        //Droite
        translate([-2,0,27.5]){
            rotate(a = [0,90,0]){
                cylinder(h = 20, r = 2.5, $fn=15);
            }    
        }
        translate([19,0,27.5]){
            sphere(r = 2.5, $fn=15);
        }
        translate([19,-20,27.5]){
            rotate(a = [0,90,90]){
                cylinder(h = 20, r = 2.5, $fn=15);
            }    
        }
        translate([19,-20,27.5]){
            sphere(r = 2.5, $fn=15);
        }
        translate([19,-20,20]){
            rotate(a = [0,0,0]){
                cylinder(h = 7.5, r = 2.5, $fn=15);
            }    
        }
        translate([19,-17.5,27.5]){
            rotate(a = [0,0,0]){
                cylinder(h = 5, r = 2, $fn=15);
            }    
        }
    }
}




/*********************************
Bécher
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le parametre remplissage sert à déterminer le remplissage du bécher en %.
    Valeurs acceptées: 0 à 100
*/
module becher(lego=true, remplissage=0){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                constructionBecher(remplissage);            
            }
            doblo(col=-1.5, row=-1.5, width=3, length=3, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        constructionBecher(remplissage);
    }    
    
    module constructionBecher(remplissage){ 
        translate([0,0,5]){
            difference(){
            union(){
            //Bas du bécher
            difference (){   
                scale([1, 1, 0.5]){    
                    sphere(r = 12);
                }
                translate([-15,-15,0]){
                    cube(size =[30,30,20]);
                }
                translate([-15,-15,-25]){
                    cube(size =[30,30,20]);
                }
            }
            
            //Corps du bécher
            difference(){
                cylinder(h = 30, r = 12);
                translate([0,0,2]){
                    cylinder(h = 30, r = 9);
                    }    
            }            
            
            //Haut du bécher (cylindre qui ressort)
            translate([0,0,30]){ 
                difference(){
                    translate([-4.5,0,-7]){
                        rotate(a=[0,-40,0]){ 
                            cylinder(h = 10, r = 5);
                        }
                        }
                    translate([0,0,4.9]){
                        cube(size =[30,30,10], center=true);   
                    }
                    translate([0,0,-15]){
                        cylinder(h = 20, r = 9);
                    } 
                    }
            }
        }//union
            //Partie pour enlever le bout arrondi qui sort du bécher
            translate([-5,0,23.2]){
                            rotate(a=[0,-40,0]){ 
                                cylinder(h = 15, r = 3);
                            }
                            }
        
        }//difference 
 
        //Remplissage du bécher
        translate([0,0,2]){            
            if(remplissage <= 100){
                remplissage = remplissage*0.01*20;
                cylinder(h = remplissage, r = 9);
            }
            if(remplissage > 100){
                remplissage = 0;
                cylinder(h = remplissage, r = 9);
            }
             if(remplissage < 0){
                remplissage = 0;
                cylinder(h = remplissage, r = 9);
            }           
        }
        
        }
    }    
}



/*********************************
Portoir tubes à essai
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre imprimer sert à placer les pièces de façon à imprimer facilement le portoir
    Valeurs acceptées: true ou false
*/
module supportTube(lego=true, imprimer=true){
    //Avec Lego
    if(lego == true){
        union(){
            translate([0,0,4]){
                piliers();
                if(imprimer == true){
                    translate([0,30,-3]){
                        rotate(a=[0,180,0]){
                            hautSupportTube(); 
                        }
                    }
                }
                if(imprimer == false){
                    translate([0,0,26]){
                        hautSupportTube(); 
                    }
                }
            }
            doblo(col=-3, row=-1.5, width=6, length=3, height=HALF, nibbles_on_off=false);
        }
    }
    //Sans Lego
    if(lego == false){
        union(){
            translate([0,0,4]){
                piliers(); 
                if(imprimer == true){
                    translate([0,30,-3]){
                        rotate(a=[0,180,0]){
                            hautSupportTube(); 
                        }
                    }
                }
                if(imprimer == false){
                    translate([0,0,26]){
                        hautSupportTube(); 
                    }
                }
            }
            block(col=-3, row=-1.5, width=6, length=3, height=HALF, nibbles_on_off=false);
        }
    }
    
    module piliers(){
        //Les piliers
        //Gauche
        translate([-18,0,0]){
            cylinder(h = 25, r = 4);
        }
        //Droite
        translate([18,0,0]){
            cylinder(h = 25, r = 4);
        }
    }
    
    module hautSupportTube(){            
        difference(){
        translate([0,0,-4]){            
            block(col=-3, row=-1.5, width=6, length=3, height=HALF, nibbles_on_off=false);
        }
        translate([0,0,-26]){
            piliers();
        }
        translate([-10,-6,-6]){
        cylinder(h = 10, r = 4);
        }
        translate([0,-6,-6]){
        cylinder(h = 10, r = 4);
        }
        translate([10,-6,-6]){
        cylinder(h = 10, r = 4);
        }

        translate([-10,6,-6]){
        cylinder(h = 10, r = 4);
        }
        translate([0,6,-6]){
        cylinder(h = 10, r = 4);
        }
        translate([10,6,-6]){
        cylinder(h = 10, r = 4);
        }        
        }
    }
}



/*********************************
Tube
**********************************/
/*
Le parametre remplissage sert à déterminer le remplissage du tube en %.
    Valeurs acceptées: 0 à 100
*/
module tube(remplissage = 0){

    translate([0,0,4]){
    //Bas du tube
    difference (){   
        scale([1, 1, 1]){    
            sphere(r = 3.75);
        }
        translate([0,0,5]){
            cube(size =[15,15,10], center = true);
        }
    }
    
    //Corps tube
    difference(){
        cylinder(h = 30, r = 3.75);
        translate([0,0,2]){
            cylinder(h = 30, r = 2.5);
        } 
    }
    
    //Remplissage du tube
        translate([0,0,2]){            
            if(remplissage <= 100){
                remplissage = remplissage*0.01*25;
                cylinder(h = remplissage, r = 2.5);
            }
            if(remplissage > 100){
                remplissage = 0;
                cylinder(h = remplissage, r = 2.5);
            }
             if(remplissage < 0){
                remplissage = 0;
                cylinder(h = remplissage, r = 2.5);
            }           
        }
    }
    
}



/*********************************
Bec bunsen
**********************************/
/*
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
*/
module becBunsen(lego=true){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                constructionBecBunsen();            
            }
            doblo(col=-1.5, row=-1.5, width=3, length=3, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        constructionBecBunsen();
    }
    
    module constructionBecBunsen(){
        union(){
            //Bas du bec busen
            cylinder(h = 3, r = 12);
            translate([0,0,3]){
                cylinder(h = 3, r1 = 12, r2 = 4);
            }
            //Tube du cylindre
            translate([0,0,5]){
                cylinder(h = 20, r = 4);
            }
            //Flame
            translate([0,0,29]){
                difference(){
                    sphere(r=5);
                    cylinder(h = 7, r = 5);    
                }
                cylinder(h = 9, r1 = 5, r2= 0);
            }
        }
    }
}



/*********************************
Ernlenmeyer
**********************************/
/*
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le parametre remplissage sert à déterminer le remplissage de l'ernlenmeyer en %.
    Valeurs acceptées: 0 à 100
*/
module ernlenmeyer(lego=true, remplissage=0){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                constructionErnlenmeyer(remplissage);            
            }
            doblo(col=-1.5, row=-1.5, width=3, length=3, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        constructionErnlenmeyer(remplissage);
    }
    
    module constructionErnlenmeyer(remplissage){        
        translate([0,0,5]){
            difference(){
            union(){
                //Bas ernlenmeyer
                difference (){   
                    scale([1, 1, 0.5]){    
                        sphere(r = 12);
                    }
                    translate([-15,-15,0]){
                        cube(size =[30,30,20]);
                    }
                    translate([-15,-15,-25]){
                        cube(size =[30,30,20]);
                    }
                }
                //Centre ernlenmeyer
                difference(){
                    union(){
                        cylinder(h = 20, r1 = 12, r2 = 4);
                        translate([0,0,19]){
                            cylinder(h = 10, r = 4);
                        }
                    }
                    cylinder(h = 40, r = 3);                    
                }
                //Haut ernlenmeyer
                translate([0,0,30]){
                    difference(){
                        sphere(r = 4.5);
                        translate([-7.5,-7.5,2]){
                            cube(size =[15,15,10]);    
                        }
                        translate([-7.5,-7.5,-11.5]){
                            cube(size =[15,15,10]);    
                        }
                        translate([0,0,-5]){
                        cylinder(h = 10, r = 3);} 
                    }
                }
            }
            }//difference
                        
            //Remplissage du ernlenmeyer
            translate([0,0,2]){            
                if(remplissage <= 100){
                    remplissage = remplissage*0.01*28;
                    cylinder(h = remplissage, r = 4);
                }
                if(remplissage > 100){
                    remplissage = 0;
                    cylinder(h = remplissage, r = 4);
                }
                 if(remplissage < 0){
                    remplissage = 0;
                    cylinder(h = remplissage, r = 4);
                }           
            }
        }
    }
}


/*********************************
Support
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre imprimer sert à placer les pièces de façon à imprimer facilement le portoir
    Valeurs acceptées: true ou false
*/
module supportChauffer(lego=true, imprimer=true){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                piliers();
                if(imprimer==true){
                    translate([0,50,-4]){
                        hautSupport(lego);
                    }
                }
                if(imprimer==false){
                    translate([0,0,45]){
                        hautSupport(lego);
                    }
                }
            }
            
            doblo(col=-2.5, row=-2.5, width=5, length=5, height=HALF, nibbles_on_off=true);
        }
    }    
    //Sans Lego
    if(lego==false){
        union(){
            translate([0,0,4]){
                piliers();
                if(imprimer==true){
                    translate([0,50,-4]){
                        hautSupport(lego);
                    }
                }
                if(imprimer==false){
                    translate([0,0,45]){
                        hautSupport(lego);
                    }
                }
            }
            block(col=-2.5, row=-2.5, width=5, length=5, height=HALF, nibbles_on_off=false);
        }
    }
    
    //Piliers
    module piliers(){
        //Les piliers
        //Haut Gauche
        translate([-16,16,0]){
            cylinder(h = 48, r = 3);
        }
        //Haut Droite
        translate([16,16,0]){
            cylinder(h = 48, r = 3);
        }
        //Bas Gauche
        translate([-16,-16,0]){
            cylinder(h = 48, r = 3);
        }
        //Bas Droite
        translate([16,-16,0]){
            cylinder(h = 48, r = 3);
        }
    }    
    //Haut du support
    module hautSupport(lego=true){
    difference(){
        if(lego == true){
            block(col=-2.5, row=-2.5, width=5, length=5, height=HALF, nibbles_on_off=true);
        }
        if(lego == false){
            block(col=-2.5, row=-2.5, width=5, length=5, height=HALF, nibbles_on_off=false);
        }
        translate([-16,16,-7]){
            cylinder(h = 10, r = 3);
        }
        //Haut Droite
        translate([16,16,-7]){
            cylinder(h = 10, r = 3);
        }
        //Bas Gauche
        translate([-16,-16,-7]){
            cylinder(h = 10, r = 3);
        }
        //Bas Droite
        translate([16,-16,-7]){
            cylinder(h = 10, r = 3);
        }
    }
    }   
    
}





/*********************************
Microscope
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre imprimer sert à placer les pièces de façon à imprimer facilement le microscope
    Valeurs acceptées: true ou false
*/
module microscope(lego=true, imprimer=true){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                baseMicroscope();
                
                if(imprimer==true){                    
                    translate([0,11,8]){
                        centreMicroscope();
                    }                  
                    translate([-40,0,-4]){
                        lampeMicroscope();
                    }
                    translate([-40,25,13.5]){
                        rotate(a = [90,180,0]){
                            stage();
                        }
                    }
                    translate([35,0,-4]){
                        tubeOcculaire();
                    }
                    translate([35,20,-2]){
                        rotate(a = [200,0,0]){
                            lentille();
                        }
                    }
                    
                }
                if(imprimer==false){
                    translate([0,11,8]){
                        centreMicroscope();
                    }
                    translate([0,-2,11]){
                        lampeMicroscope();
                    }
                    translate([0,-6,29]){
                        stage();
                    }
                    translate([0,0,47]){
                        tubeOcculaire();
                    }
                    translate([0,0,47]){
                        lentille();
                    }
                }
                
            }
            
            doblo(col=-2.5, row=-4, width=5, length=7, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        union(){            
            baseMicroscope();
            if(imprimer==true){                
                translate([0,11,8]){
                    centreMicroscope();
                }                
                translate([-40,0,0]){
                    lampeMicroscope();
                }
                translate([-40,25,17.5]){
                    rotate(a = [90,180,0]){
                        stage();
                    }
                }
                translate([35,0,0]){
                    tubeOcculaire();
                }
                translate([35,20,2]){
                    rotate(a = [200,0,0]){
                        lentille();
                    }
                }
            }
            if(imprimer==false){
                translate([0,11,8]){
                    centreMicroscope();
                }
                translate([0,-2,11]){
                    lampeMicroscope();
                }
                translate([0,-6,29]){
                    stage();
                }
                translate([0,0,47]){
                    tubeOcculaire();
                }
                translate([0,0,47]){
                    lentille();
                }
            }            
        }
    }
    
    //Bas du microscope
    module baseMicroscope(){
        difference(){
            scale (v=[1,1.5,1]){
                cylinder(h = 8, r = 20);
            }
            translate([0,-15,-1]){
                scale (v=[1,1.5,1]){
                    cylinder(h = 10, r = 14);
                }
            }    
        }
    }
    
    //Centre du microscope
    module centreMicroscope(){
        difference(){
            union(){
                //Tronc
                translate([-4,0,30]){
                rotate(a = [90,0,90]){
                    difference(){
                        scale (v=[1,1.5,1]){
                            cylinder(h = 8, r = 20);
                        }
                        translate([-15,0,-1]){
                            scale (v=[1.5,1.2,1]){
                                cylinder(h = 10, r = 14);
                            }
                        }
                        translate([-15,0,0]){
                            cube(size =[20,70,25], center = true); 
                        }
                    }    
                }
                }
                //Parties latérales du tronc
                translate([3.5,0,2]){
                    rotate(a = [90,0,90]){
                    hull(){
                        translate([0,0,0]){ cylinder(r=2,h=3);}
                        translate([14,0,0]){ cylinder(r=2,h=3);}
                        translate([0,14,0]){ cylinder(r=2,h=3);}
                    }
                    }
                }
                translate([-7,0,2]){
                    rotate(a = [90,0,90]){
                    hull(){
                        translate([0,0,0]){ cylinder(r=2,h=3);}
                        translate([14,0,0]){ cylinder(r=2,h=3);}
                        translate([0,14,0]){ cylinder(r=2,h=3);}
                    }
                    }
                }
                //Boutons
                translate([3,0,54]){
                    rotate(a = [90,0,90]){
                        cylinder(h = 4, r = 4.5);
                    }    
                }
                translate([3,5,45]){
                    rotate(a = [90,0,90]){
                        cylinder(h = 4, r1 = 3, r2 = 2.5);
                    }    
                }
                translate([5,5,45]){
                    rotate(a = [90,0,90]){
                        cylinder(h = 4, r = 1.5);
                    }    
                }
            }
            //Trous pour joindre les autres pièces
            //Pour lampe
            translate([0,0,5]){
                rotate(a = [90,0,0]){
                    cylinder(h = 7, r = 2.1, $fn=30);
                }
            //Pour stage
            translate([0,-2,5]){
                cylinder(h = 8, r = 2.1, $fn=30);
            }
            //Pour tube occulaire
            translate([0,-2,51]){
                rotate(a = [90,0,0]){
                    cylinder(h = 5, r = 2.1, $fn=30);
                }
            }
            translate([0,-2,43]){
                rotate(a = [90,0,0]){
                    cylinder(h = 5, r = 2.1, $fn=30);
                }
            }
        }
        }
        
    }
    
    //Lampe Microscope
    module lampeMicroscope(){
        //Support Lampe    
        difference(){        
            cylinder(h = 4, r = 9);        
            translate([0,-5,-1]){
                cylinder(h = 6, r = 6);
            }
        }  
        //Lampe
        translate([0,-6,1]){    
            rotate(a = [9,0,0]){
                difference(){
                    cylinder(h = 4, r = 7);
                    translate([0,0,3]){
                        cylinder(h = 2, r = 5);
                    }
                }
            }
        }
        //Joint
        translate([0,13,2]){
            rotate(a = [90,0,0]){
                cylinder(h = 5, r = 2, $fn=30);
            }
        }        
    }
    
    //Stage Microscope
    module stage(){
        union(){
            //Stage
            cube(size =[35,35,5], center = true);
            //clips
            translate([10,0,3]){
                cube(size =[4,10,2], center = true);
            }
            translate([-10,0,3]){
                cube(size =[4,10,2], center = true);
            }
            //verre ovale
            translate([0,0,-2]){
                cylinder(h = 5, r = 5);
            }
            translate([0,0,-1.5]){
                cylinder(h = 5, r = 3);
            }
            //Joint
            translate([0,15,-10.5]){
                cylinder(h = 8, r = 2, $fn=30);        
            }
        }
    }
    
    //Tube occulaire microscope
    module tubeOcculaire(){
        difference(){/////////////////////////////////
        union(){
            //Tube occulaire
            translate([0,0,20]){
                difference(){
                    cylinder(h = 18, r = 4);
                    translate([0,0,17]){
                        cylinder(h = 5, r = 3);
                    }    
                }
            }        
            cylinder(h = 20, r = 6);
            //Joints
            translate([0,9,8]){
                rotate(a = [90,0,0]){
                    cylinder(h = 4, r = 2, $fn=30);
                }
            }
            translate([0,9,16]){
                rotate(a = [90,0,0]){
                    cylinder(h = 4, r = 2, $fn=30);
                }
            }           
        }        
//////////////////
        //Trou pour joint
        translate([0,0,-1]){
            rotate(a = [-20,0,0]){
                cylinder(h = 3, r = 2.1, $fn=30);
            }
        }
//////////////////
        }////////////// 
    }
    
    //Lentille microscope
    module lentille(){
        difference(){
            union(){
                //Rond changer lentille
                translate([0,-6,0]){
                    rotate(a = [-20,0,0]){
                        cylinder(h = 4, r = 10);    
                        translate([0,0,-2]){
                            cylinder(h = 4, r1 = 8, r2= 10);
                        }    
                    }
                }
                //Lentilles
                translate([0,-4,-13]){
                    cylinder(h = 14, r = 2.5);
                }
                translate([6,-13,-7]){
                    rotate(a = [-30,0,30]){
                        cylinder(h = 10, r = 2.5);
                    }
                }
                translate([-6,-13,-8]){
                    rotate(a = [-30,0,-30]){
                        cylinder(h = 10, r = 2.5);
                    }
                }
            }            
            cylinder(h = 20, r = 6);            
        }
////////////////////        
        //Joint
        rotate(a = [-20,0,0]){
            cylinder(h = 2, r = 2, $fn=30);
        }
///////////////////        
    }
}



/*********************************
Ballon sphérique
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre remplissage sert à déterminer le remplissage du ballon sphérique en %.
    Valeurs acceptées: 0 à 100
Le paramètre tube2 sert à déterminer si on met un deuxième tube ou pas
    Valeurs acceptées: true ou false
Le paramètre tube3 sert à déterminer si on met un troisième tube ou pas
    Valeurs acceptées: true ou false
*/
module ballonSpherique(lego=true, remplissage=0, tube2=false, tube3=false){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                constructionBallon(remplissage, tube2, tube3);            
            }
            doblo(col=-1.5, row=-1.5, width=3, length=3, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        constructionBallon(remplissage, tube2, tube3);
    }
    
    module constructionBallon(remplissage, tube2, tube3){
        difference(){
            union(){
                //Rond
                difference(){
                    translate([0,0,10]){
                        sphere(r = 12);
                    }
                    translate([0,0,-5]){
                        cube(size =[30,30,10], center=true);
                    }
                }
                //Tube
                translate([0,0,20]){
                    cylinder(h = 15, r = 4);
                }
                //Haut ballon                                   
                translate([0,0,35]){
                    difference(){
                        sphere(r = 4.5);
                        translate([-7.5,-7.5,2]){
                            cube(size =[15,15,10]);    
                        }
                        translate([-7.5,-7.5,-11.5]){
                            cube(size =[15,15,10]);    
                        }
                        translate([0,0,-5]){
                            cylinder(h = 10, r = 3);
                        } 
                    }
                }
                //2ème tube
                if(tube2==true){                     
                    rotate(a=[0,-22,0]){
                        translate([0,0,33]){
                            translate([0,0,-15]){
                                difference(){
                                    cylinder(h = 15, r = 4);
                                    translate([0,0,-5]){
                                        cylinder(h = 25, r = 3);
                                    }
                                }
                            }
                            difference(){
                                sphere(r = 4.5);
                                translate([-7.5,-7.5,2]){
                                    cube(size =[15,15,10]);    
                                }
                                translate([-7.5,-7.5,-11.5]){
                                    cube(size =[15,15,10]);    
                                }
                                translate([0,0,-5]){
                                    cylinder(h = 10, r = 3);
                                } 
                            }
                        }
                    }
                }
                //3ème tube
                if(tube3==true){                     
                    rotate(a=[0,22,0]){
                        translate([0,0,33]){
                            translate([0,0,-15]){
                                difference(){
                                    cylinder(h = 15, r = 4);
                                    translate([0,0,-5]){
                                        cylinder(h = 25, r = 3);
                                    }
                                }
                            }
                            difference(){
                                sphere(r = 4.5);
                                translate([-7.5,-7.5,2]){
                                    cube(size =[15,15,10]);    
                                }
                                translate([-7.5,-7.5,-11.5]){
                                    cube(size =[15,15,10]);    
                                }
                                translate([0,0,-5]){
                                    cylinder(h = 10, r = 3);
                                } 
                            }
                        }
                    }
                }
                
            }
            //Partie vide           
            translate([0,0,3]){
                cylinder(h = 40, r = 3);
            }  
        }//difference
        
        //Remplissage du ballon sphérique
        translate([0,0,2]){            
            if(remplissage <= 100){
                remplissage = remplissage*0.01*33;
                cylinder(h = remplissage, r = 4);
            }
            if(remplissage > 100){
                remplissage = 0;
                cylinder(h = remplissage, r = 4);
            }
             if(remplissage < 0){
                remplissage = 0;
                cylinder(h = remplissage, r = 4);
            }           
        }
        
    }
}





/*********************************
Fiole
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre remplissage sert à déterminer le remplissage de la fiole en %.
    Valeurs acceptées: 0 à 100
*/
module fiole(lego=true, remplissage=0){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                constructionFiole(remplissage);            
            }
            doblo(col=-1.5, row=-1.5, width=3, length=3, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        constructionFiole(remplissage);
    }
    
    module constructionFiole(remplissage){        
        translate([0,0,5]){
            difference(){
            union(){
                //Bas fiole
                difference (){   
                    scale([1, 1, 0.6]){    
                        sphere(r = 12);
                    }
                    translate([-15,-15,5]){
                        cube(size =[30,30,20]);
                    }
                    translate([-15,-15,-25]){
                        cube(size =[30,30,20]);
                    }
                }                
                //Centre fiole
                translate([0,0,5]){
                    cylinder(h = 3, r1 = 8.5, r2 = 4);
                }
                translate([0,0,8]){
                    cylinder(h = 22, r = 4);
                }
                //Haut fiole
                translate([0,0,30]){
                    difference(){
                        sphere(r = 4.5);
                        translate([-7.5,-7.5,2]){
                            cube(size =[15,15,10]);    
                        }
                        translate([-7.5,-7.5,-11.5]){
                            cube(size =[15,15,10]);    
                        }
                        translate([0,0,-5]){
                        cylinder(h = 10, r = 3);}
                    }
                }                
            }
                translate([0,0,-2]){
                    cylinder(h = 40, r = 3);
                }
            }//difference
                        
            //Remplissage du ernlenmeyer
            translate([0,0,2]){            
                if(remplissage <= 100){
                    remplissage = remplissage*0.01*28;
                    cylinder(h = remplissage, r = 4);
                }
                if(remplissage > 100){
                    remplissage = 0;
                    cylinder(h = remplissage, r = 4);
                }
                 if(remplissage < 0){
                    remplissage = 0;
                    cylinder(h = remplissage, r = 4);
                }           
            }
        }
    }
}



/*********************************
Éprouvette
**********************************/
/* 
Le paramètre lego sert à déterminer si on met une base lego ou pas
    Valeurs acceptées: true ou false
Le paramètre remplissage sert à déterminer le remplissage de l'éprouvette en %.
    Valeurs acceptées: 0 à 100
*/
module eprouvette(lego=true, remplissage=0){
    //avec Lego
    if(lego==true){
        union(){
            translate([0,0,4]){
                constructionEprouvette(remplissage);            
            }
            doblo(col=-1.5, row=-1.5, width=3, length=3, height=HALF, nibbles_on_off=false);
        }
    }    
    //Sans Lego
    if(lego==false){
        constructionEprouvette(remplissage);
    }
    
    module constructionEprouvette(remplissage){
        difference(){
            union(){
                //Socle
                cylinder(h = 3, r = 12);
                //Éprouvette       
                translate([0,0,3]){
                    cylinder(h = 34, r = 7);
                }        
                translate([-4,0,33]){
                    rotate(a=[0,-40,0]){
                        cylinder(h = 5, r = 3);                   
                    }
                }
            }//union
            translate([0,0,2]){
                cylinder(h = 40, r = 5);
            }
            translate([-7.5,-7.5,36.9]){
                cube(size =[15,15,10]);    
            }
            translate([-3.5,0,33]){
                rotate(a=[0,-40,0]){
                        cylinder(h = 7, r = 2);
                }
            }
            translate([-4,0,36]){
                rotate(a=[0,40,0]){
                    cube(size =[5,4,3], center=true);
                }
            }
        }//difference
        
        //Remplissage du ernlenmeyer
            translate([0,0,2]){            
                if(remplissage <= 100){
                    remplissage = remplissage*0.01*31;
                    cylinder(h = remplissage, r = 5);
                }
                if(remplissage > 100){
                    remplissage = 0;
                    cylinder(h = remplissage, r = 5);
                }
                 if(remplissage < 0){
                    remplissage = 0;
                    cylinder(h = remplissage, r = 5);
                }           
            }
    }
}



/*********************************
Entonnoir
**********************************/
/* 
Le paramètre imprimer sert à placer les pièces de façon à imprimer facilement l'entonnoir
    Valeurs acceptées: true ou false
*/
module entonnoir(imprimer=true){
    if(imprimer == false){
        constructionEntonnoir();
    }
    if(imprimer == true){
        rotate(a=[0,180,0]){
            translate([0,0,-20]){
                constructionEntonnoir();
            }
        }
    }
    
    
    module constructionEntonnoir(){
        difference(){
            union(){
                //Entonnoir forme
                cylinder(h = 10, r = 2.8, $fn=15);
                translate([0,0,10]){
                    cylinder(h = 10, r1 = 2.8, r2 = 12);
                }
            }
            //Vidage entonnoir
            translate([0,0,10]){
                cylinder(h = 11, r1 = 0, r2 = 10);
            }
            translate([0,0,-1]){
                cylinder(h = 15, r = 1.3, $fn=15);
            }
        }
    }
}

